/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2012, Systemyde International Corporation, ALL RIGHTS RESERVED          **/
/**                                                                                       **/
/** timer chip core                                                   Rev 0.0  12/20/2012 **/
/**                                                                                       **/
/*******************************************************************************************/
module tim_core (data_drv, data_out, isa_drv, fi_drv, fi_out, data_bus, isa_bus, osc_in, ph1,
                 ph2, pwr_up, pwo, sync);

  input         data_bus;      /* memory/peripheral data bus                          bus  */
  input         isa_bus;       /* instruction address/data bus                        bus  */
  input         osc_in;        /* 32768Hz clock input                                 brd  */
  input         ph1;           /* clock phase 1                                       bus  */
  input         ph2;           /* clock phase 2                                       bus  */
  input         pwr_up;        /* power on reset                                      brd  */
  input         pwo;           /* power off                                           bus  */
  input         sync;          /* synchronization                                     bus  */

  output        data_drv;      /* memory/peripheral data bus                          bus  */
  output        data_out;      /* memory/peripheral data bus                          bus  */
  output        isa_drv;       /* instruction address/data bus                        bus  */
  output        fi_drv;        /* flag bus                                            bus  */
  output        fi_out;        /* flag bus                                            bus  */

  /*****************************************************************************************/
  /*                                                                                       */
  /* signal declarations                                                                   */
  /*                                                                                       */
  /*****************************************************************************************/
  wire        a_hz100,      b_hz100;                       /* gated 100Hz                  */
  wire        aalrm_nxt,    balrm_nxt;                     /* alarm register input         */
  wire        acc_nxt;                                     /* accuracy register input      */
  wire        acctmr_cry;                                  /* accuracy inc carry           */
  wire        acctmr_inc;                                  /* accuracy inc next            */
  wire        acctmr_tc;                                   /* accuracy terminal count      */
  wire        acctmrdadj_cry;                              /* accuracy dec adj carry       */
  wire        aclk_0cry,    bclk_0cry;                     /* clock inc force carry in 0   */
  wire        aclk_1cry,    bclk_1cry;                     /* clock inc force carry in 1   */
  wire        aclk_cry,     bclk_cry;                      /* clock inc carry out          */
  wire        aclk_cry_in,  bclk_cry_in;                   /* clock inc carry in           */
  wire        aclk_en,      bclk_en;                       /* enable clock register        */
  wire        aclk_inc,     bclk_inc;                      /* clock inc next               */
  wire        aclk_mat,     bclk_mat;                      /* clock alarm match            */
  wire        aclkdadj_cry, bclkdadj_cry;                  /* clock dec adj carry          */
  wire        adjust_pls;                                  /* synced adjust pulse          */
  wire        alma_res,     almb_res;                      /* alarm reset                  */
  wire        ascr_nxt,     bscr_nxt;                      /* scratch register input       */
  wire        astatus_out,  bstatus_out;                   /* status reg read data         */
  wire        auto_adj;                                    /* enable atimer                */
  wire        c10240_pls;                                  /* 10240 Hz clock               */
  wire        c5600_pls;                                   /* 5600 Hz clock                */
  wire        correct_out;                                 /* accuracy reg read data       */
  wire        daddc_pls;                                   /* dadd=c inst pulse            */
  wire        data_drv, data_out;                          /* memory/peripheral data bus   */
  wire        dtza_res,     dtzb_res;                      /* dtz reset                    */
  wire        dtzit_res;                                   /* dtzit reset                  */
  wire        en_aalrm,     en_balrm;                      /* enable alarm register        */
  wire        en_acc;                                      /* enable accuracy register     */
  wire        en_accmatch;                                 /* accuracy match enable        */
  wire        en_acctmr;                                   /* enable accuracy timer inc    */
  wire        en_amatch,    en_bmatch;                     /* alarm match enable           */
  wire        en_ascr,      en_bscr;                       /* enable scratch register      */
  wire        en_itmr;                                     /* enable interval timer inc    */
  wire        en_tmatch;                                   /* interval timer match enable  */
  wire        en_tmr;                                      /* enable interval timer reg    */
  wire        fi_drv, fi_out;                              /* flag bus                     */
  wire        hz100_pls;                                   /* synced 100Hz pulse           */
  wire        isa_drv;                                     /* instr addr/data bus          */
  wire        itmr_cry;                                    /* itimer inc carry             */
  wire        itmr_inc;                                    /* itimer inc next              */
  wire        itmr_tc;                                     /* itimer terminal count        */
  wire        itmrdadj_cry;                                /* itimer dec adj carry         */
  wire        ld_cycle;                                    /* next cycle                   */
  wire        ld_iftch;                                    /* load serial inst             */
  wire        ld_inst;                                     /* latch instruction            */
  wire        pfadc_pls;                                   /* pfad=c inst pulse            */
  wire        ph2m;                                        /* muxed phase 2 clock          */
  wire        pus_res;                                     /* pus reset                    */
  wire        rd_aclk,      rd_bclk;                       /* gated read clk               */
  wire        rd_ahclk,     rd_bhclk;                      /* gated read/hold clk          */
  wire        rd_aonly,     rd_bonly;                      /* gated read clk only          */
  wire        rdacc_inst;                                  /* read factor inst             */
  wire        rdalrm_inst;                                 /* read alarm inst              */
  wire        rdclk_inst;                                  /* read clock inst              */
  wire        rdhclk_inst;                                 /* read/hold clock inst         */
  wire        rdscr_inst;                                  /* read scratch inst            */
  wire        rdtmr_inst;                                  /* read interval timer inst     */
  wire        srvc_drv;                                    /* service request drv enable   */
  wire        srvc_req;                                    /* service request              */
  wire        start_idl;                                   /* trigger idle state           */
  wire        start_run;                                   /* trigger run state            */
  wire        state_e, state_c, state_8, state_0;          /* decoded states               */
  wire        state_1, state_6, state_d, state_b;
  wire        state_c80;                                   /* states c-8-0                 */
  wire        state_ec;                                    /* states e-c                   */
  wire        state_ec80;                                  /* states e-c-8-0               */
  wire        state_ec801;                                 /* states e-c-8-0-1             */
  wire        tmr_nxt;                                     /* itimer reg input             */
  wire        wr_aclk,      wr_bclk;                       /* gated write clk              */
  wire        wr_acclk,     wr_bcclk;                      /* gated write/correct clk      */
  wire        wracc_inst;                                  /* write accuracy inst          */
  wire        wralm_inst;                                  /* write en/dis alarm inst      */
  wire        wralrm_inst;                                 /* write alarm inst             */
  wire        wrcclk_inst;                                 /* write/correct clock inst     */
  wire        wrclk_inst;                                  /* write clock inst             */
  wire        writim_inst;                                 /* write itimer inst            */
  wire        wrptr_inst;                                  /* write pointer inst           */
  wire        wrscr_inst;                                  /* write scratch inst           */
  wire        wrstat_inst;                                 /* write status inst            */
  wire        wrstst_inst;                                 /* write start/stop inst        */
  wire        wrtmr_inst;                                  /* write interval timer inst    */
  wire  [2:0] acctmr_nxt;                                  /* accuracy next                */
  wire  [2:0] aclk_nxt,     bclk_nxt;                      /* clock reg next               */
  wire  [2:0] itmr_nxt;                                    /* itimer next                  */

  reg         accmatch_reg;                                /* accuracy match compare       */
  reg         acctmr_cry_reg;                              /* accuracy latched carry bit   */
  reg         acctmrtc_reg;                                /* accuracy latched term cnt    */
  reg         aclk_cry_reg, bclk_cry_reg;                  /* clock latched carry bit      */
  reg         aclkhld_reg,  bclkhld_reg;                   /* hold increment op            */
  reg         aclkinc2_reg, bclkinc2_reg;                  /* increment by 2               */
  reg         adj_dn_reg;                                  /* adjust prescaler down        */
  reg         adj_up_reg;                                  /* adjust prescaler up          */
  reg         adjlat_reg;                                  /* adjust synced                */
  reg         alaen_reg,    alben_reg;                     /* alarm enable register        */
  reg         alma_reg,     almb_reg;                      /* alarm match                  */
  reg         amatch_reg,   bmatch_reg;                    /* alarm match compare          */
  reg         awake_reg;                                   /* in sync with awake cpu       */
  reg         c100_pls;                                    /* 100 Hz clock                 */
  reg         c16384_pls;                                  /* 16384 Hz clock               */
  reg         c100lat_reg;                                 /* 100Hz synced                 */
  reg         ckaen_reg,    ckben_reg;                     /* clock enable register        */
  reg         cregn_reg, regnc_reg;                        /* latched inst decodes         */
  reg         daddc_inst;                                  /* dadd=c instruction           */
  reg         daddc_mis;                                   /* dadd=c tracker               */
  reg         data_cha,     data_chb;                      /* read data mux output         */
  reg         data_reg;                                    /* ph1 sampled data signal      */
  reg         dindec_mis;                                  /* din tracker                  */
  reg         dtza_reg,     dtzb_reg;                      /* clock dec thru zero          */
  reg         dtzit_reg;                                   /* interval tmr dec thru zero   */
  reg         enabl_reg;                                   /* chip enable                  */
  reg         hz100_reg;                                   /* 10mS time tick               */
  reg         isa_reg;                                     /* ph1 sampled isa signal       */
  reg         iten_reg;                                    /* timer enable register        */
  reg         itmr_cry_reg;                                /* itimer latched carry bit     */
  reg         itmrtc_reg;                                  /* itimer latched term cnt      */
  reg         pfadc_inst;                                  /* pfad=c instruction           */
  reg         pfadc_mis;                                   /* pfad=c tracker               */
  reg         ptr_reg;                                     /* pointer register             */
  reg         pus_reg;                                     /* power-up status              */
  reg         sync_dly;                                    /* delayed sync signal          */
  reg         sync_reg;                                    /* ph1 sampled sync signal      */
  reg         t0_reg, t1_reg;                              /* bit identifiers              */
  reg         tmatch_reg;                                  /* interval tmr match compare   */
  reg         updown_reg;                                  /* latched adj dir select       */
  reg   [1:0] idl_reg;                                     /* idle state tracker           */
  reg   [1:0] pwo_reg;                                     /* ph2 sampled pwo signal       */
  reg   [1:0] run_reg;                                     /* run state tracker            */
  reg   [2:0] adjsyn_reg;                                  /* adjust synchronization       */
  reg   [2:0] c100syn_reg;                                 /* 100Hz synchronizer           */
  reg   [2:0] c10240_reg;                                  /* 10240 Hz divider             */
  reg   [2:0] acctmrdadj_out;                              /* accuracy dec adj output      */
  reg   [2:0] aclkdadj_out, bclkdadj_out;                  /* clock dec adj output         */
  reg   [2:0] itmrdadj_out;                                /* itimer dec adj output        */
  reg   [3:0] cycle_reg;                                   /* clock cycle                  */
  reg   [3:0] daddc_reg;                                   /* dadd=c tracker               */
  reg   [3:0] dindec_reg;                                  /* din tracker                  */
  reg   [3:0] pfadc_reg;                                   /* pfad=c tracker               */
  reg   [3:0] state_reg;                                   /* machine state                */
  reg   [5:0] c100_reg;                                    /* 100 Hz divider               */
  reg   [5:0] c5600_reg;                                   /* 5600 Hz divider              */
  reg   [7:0] din_reg;                                     /* address latch                */
  reg   [9:0] iftch_reg;                                   /* instruction assembly         */
  reg   [9:6] lftch_reg;                                   /* latched instruction          */
  reg  [12:0] acc_reg;                                     /* accuracy register            */
  reg  [15:0] acctmr_reg;                                  /* accuracy time register       */
  reg  [19:0] itmr_reg;                                    /* time register                */
  reg  [19:0] tmr_reg;                                     /* interval timer register      */
  reg  [55:0] aclk_reg,     bclk_reg;                      /* clock register               */
  reg  [55:0] aalrm_reg,    balrm_reg;                     /* alarm register               */
  reg  [55:0] ascr_reg,     bscr_reg;                      /* scratch register             */

  /*****************************************************************************************/
  /*                                                                                       */
  /* sample bus signals                                                                    */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph1 or negedge pwr_up) begin
    if (!pwr_up) begin
      data_reg <= 1'b0;
      isa_reg  <= 1'b0;
      sync_reg <= 1'b0;
      end
    else begin
      data_reg <= data_bus;
      isa_reg  <= isa_bus;
      sync_reg <= sync;
      end
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* instruction register and decodes                                                      */
  /*                                                                                       */
  /*****************************************************************************************/
  assign ld_cycle = cycle_reg[3];
  assign ld_iftch = sync_reg || state_6 || state_d || (state_b && |cycle_reg[1:0]);
  assign ld_inst  = state_b && cycle_reg[3];

  always @ (posedge ph2m or negedge pwo) begin
    if          (!pwo) iftch_reg <= 10'h000;
    else if (ld_iftch) iftch_reg <= {(sync_reg && isa_reg), iftch_reg[9:1]};
    end

  always @ (posedge ph2m or negedge pwo) begin
    if (!pwo) begin
      cregn_reg <= 1'b0;
      lftch_reg <= 4'h0;
      regnc_reg <= 1'b0;
      end
    else if (ld_inst) begin
      cregn_reg <= enabl_reg && ~|(iftch_reg[5:0] ^ 6'b111000);
      lftch_reg <= iftch_reg[9:6];
      regnc_reg <= enabl_reg && ~|(iftch_reg[5:0] ^ 6'b101000);
      end
    end

  assign wrstat_inst = regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0011) && ptr_reg;
  assign wrptr_inst  = regnc_reg && ~|(lftch_reg[9:7] ^ 3'b111);

  assign alma_res    = wrstat_inst && !din_reg[0];
  assign dtza_res    = wrstat_inst && !din_reg[1];
  assign almb_res    = wrstat_inst && !din_reg[2];
  assign dtzb_res    = wrstat_inst && !din_reg[3];
  assign dtzit_res   = wrstat_inst && !din_reg[4];
  assign pus_res     = wrstat_inst && !din_reg[5];

  /*****************************************************************************************/
  /*                                                                                       */
  /* data input latch                                                                      */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2m or negedge pwo) begin
    if          (!pwo) din_reg <= 8'h00;
    else if (state_ec) din_reg <= {data_reg, din_reg[7:1]};
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* pointer register                                                                      */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2m or negedge pwr_up) begin
    if                    (!pwr_up) ptr_reg <= 1'b0;
    else if (ld_inst && wrptr_inst) ptr_reg <= lftch_reg[6];
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* power up status bit                                                                   */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2m or negedge pwr_up) begin
    if      (!pwr_up) pus_reg <= 1'b1;
    else if (ld_inst) pus_reg <= !pus_res && pus_reg;
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* clock multiplexer: ph2 during run, c5600_pls during idle                              */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2 or negedge pwr_up) begin
    if (!pwr_up) begin
      pwo_reg  <= 2'b00;
      sync_dly <= 1'b0;
      end
    else begin
      pwo_reg  <= {pwo, pwo_reg[1]};
      sync_dly <= sync_reg;
      end
    end

  assign start_idl = state_b && cycle_reg[2];
  assign start_run = !sync && sync_dly;

  always @ (posedge ph2 or negedge pwr_up) begin
    if (!pwr_up) run_reg <= 2'b01;
    else         run_reg <= {(idl_reg[1] || !run_reg[0] || (run_reg[1] && !start_run)),
                             !idl_reg[1]};
    end

  always @ (posedge c5600_pls or negedge pwr_up) begin
    if (!pwr_up) idl_reg <= 2'b01;
    else         idl_reg <= {(!pwo_reg[0] || !idl_reg[0]),
                              (pwo_reg[0] && (start_idl || idl_reg[0]))};
    end

  assign ph2m = (ph2 || !pwo_reg[0] || run_reg[1] || idl_reg[1]) &&
                (c5600_pls || idl_reg[0]);

  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) awake_reg <= 1'b0;
    else         awake_reg <= pwo_reg[0] && !run_reg[1] && !idl_reg[1];
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* bus state machine                                                                     */
  /*                                                                                       */
  /*****************************************************************************************/
  /*                                                                                       */
  /*   state_reg     eeeecccc88880000111122225555aaaa4444999933336666ddddbbbb              */
  /*   cycle_reg     12481248124812481248124812481248124812481248124812481248              */
  /*   ld_cycle      ...1...1...1...1...1...1...1...1...1...1...1...1...1...1              */
  /*   ld_inst       .......................................................1              */
  /*   sync          ............................................1111111111..              */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) cycle_reg <= 4'b0100;
    else         cycle_reg <= {(!cycle_reg[3] && cycle_reg[2]),
                               (~|cycle_reg[3:2] && cycle_reg[1]),
                                ~|cycle_reg[3:1], cycle_reg[3]};
    end

  always @ (posedge ph2m or negedge pwr_up) begin
    if       (!pwr_up) state_reg <= 4'b1011;
    else if (ld_cycle) state_reg <= {(state_reg[2] || state_b), state_reg[1:0],
                                     ((state_reg[0] ^ !state_reg[3]) && !state_b)};
    end

  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) begin
      t0_reg <= 1'b0;
      t1_reg <= 1'b0;
      end
    else begin
      t0_reg <= state_b && cycle_reg[3];
      t1_reg <= state_e && cycle_reg[0];
      end
    end

  assign state_e =  state_reg[3] &&  state_reg[2] &&  state_reg[1] && !state_reg[0];
  assign state_c =  state_reg[3] &&  state_reg[2] && !state_reg[1] && !state_reg[0];
  assign state_8 =  state_reg[3] && !state_reg[2] && !state_reg[1] && !state_reg[0];
  assign state_0 = !state_reg[3] && !state_reg[2] && !state_reg[1] && !state_reg[0];
  assign state_1 = !state_reg[3] && !state_reg[2] && !state_reg[1] &&  state_reg[0];
  assign state_6 = !state_reg[3] &&  state_reg[2] &&  state_reg[1] && !state_reg[0];
  assign state_d =  state_reg[3] &&  state_reg[2] && !state_reg[1] &&  state_reg[0];
  assign state_b =  state_reg[3] && !state_reg[2] &&  state_reg[1] &&  state_reg[0];

  assign state_c80   = state_c || state_8 || state_0;
  assign state_ec    = &state_reg[3:2] && !state_reg[0];
  assign state_ec80  = state_e || state_c || state_8 || state_0;
  assign state_ec801 = state_e || state_c || state_8 || state_0 || state_1;

  /*****************************************************************************************/
  /*                                                                                       */
  /* data input decode 0xfb                                                                */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (dindec_reg or data_reg) begin
    casex ({dindec_reg, data_reg})
      5'b000x1,
      5'b00100,
      5'b00111,
      5'b01xx1: dindec_mis = 1'b0;                         /* match                        */
      default:  dindec_mis = 1'b1;                         /* mismatch                     */
      endcase
    end

  always @ (posedge ph2 or negedge pwo) begin
    if                     (!pwo) dindec_reg <= 4'b0000;
    else if (ld_inst || state_ec) dindec_reg <= (ld_inst)    ? 4'b0000 :
                                                (dindec_mis) ? 4'b1111 : dindec_reg + 1'b1;
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* isa input decode: dadd=c (0x270) and pfad=c_inst (0x3f0)                              */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (daddc_reg or isa_reg) begin
    casex ({daddc_reg, isa_reg})
      5'b00xx0,
      5'b010x1,
      5'b01101,
      5'b01110,
      5'b10000,
      5'b10011: daddc_mis = 1'b0;                          /* match                        */
      default:  daddc_mis = 1'b1;                          /* mismatch                     */
      endcase
    end

  always @ (pfadc_reg or isa_reg) begin
    casex ({pfadc_reg, isa_reg})
      5'b00xx0,
      5'b01xx1,
      5'b100x1: pfadc_mis = 1'b0;                          /* match                        */
      default:  pfadc_mis = 1'b1;                          /* mismatch                     */
      endcase
    end

  always @ (posedge ph2 or negedge pwo) begin
    if (!pwo) begin
      daddc_reg <= 4'b0000;
      pfadc_reg <= 4'b0000;
      end
    else if (ld_inst || sync_reg) begin
      daddc_reg <= (ld_inst)   ? 4'b0000 :
                   (daddc_mis) ? 4'b1111 : daddc_reg + 1'b1;
      pfadc_reg <= (ld_inst)   ? 4'b0000 :
                   (pfadc_mis) ? 4'b1111 : pfadc_reg + 1'b1;
      end
    end

  always @ (posedge ph2 or negedge pwo) begin
    if (!pwo) begin
      daddc_inst <= 1'b0;
      pfadc_inst <= 1'b0;
      end
    else if (ld_inst) begin
      daddc_inst <= ~|(daddc_reg ^ 4'b1010);
      pfadc_inst <= ~|(pfadc_reg ^ 4'b1010);
      end
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* chip enable decode                                                                    */
  /*                                                                                       */
  /*****************************************************************************************/
  assign daddc_pls = ld_inst && daddc_inst;
  assign pfadc_pls = ld_inst && pfadc_inst;

  always @ (posedge ph2 or negedge pwo) begin
    if                        (!pwo) enabl_reg <= 1'b0;
    else if (daddc_pls || pfadc_pls) enabl_reg <= pfadc_pls && ~|(dindec_reg ^ 4'b1000);
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* clock dividers                                                                        */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge osc_in or negedge pwr_up) begin
    if (!pwr_up) c16384_pls <= 1'b0;
    else         c16384_pls <= !c16384_pls;
    end

  always @ (posedge c16384_pls or negedge pwr_up) begin
    if (!pwr_up) c10240_reg <= 3'b000;
    else         c10240_reg <= c10240_reg + 1'b1;
    end

  assign c10240_pls = !c16384_pls && (c10240_reg[0] || (c10240_reg[2:1] && adj_up_reg) ||
                                     (!c10240_reg[2] && c10240_reg[1] && !adj_dn_reg));

  always @ (posedge c10240_pls or negedge pwr_up) begin
    if (!pwr_up) c5600_reg <= 6'b000000;
    else         c5600_reg <= c5600_reg + 1'b1;
    end

  assign c5600_pls = !c10240_pls && (c5600_reg[0] || (|c5600_reg[5:4] && ~|c5600_reg[3:0]));

  always @ (posedge c5600_pls or negedge pwr_up) begin
    if (!pwr_up) c100_reg <= 6'b000000;
    else         c100_reg <= (~|(c100_reg ^ 6'b110111)) ? 6'b000000 : (c100_reg + 1'b1);
    end

  always @ (negedge c5600_pls or negedge pwr_up) begin
    if (!pwr_up) c100_pls <= 1'b0;
    else         c100_pls <= c100_reg[5];
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* clock domain crossing                                                                 */
  /*                                                                                       */
  /*****************************************************************************************/
  assign adjust_pls = !adjsyn_reg[1] && adjsyn_reg[0];

  always @ (posedge c16384_pls or negedge pwr_up) begin
    if (!pwr_up) begin
      adj_dn_reg <= 1'b0;
      adj_up_reg <= 1'b0;
      adjsyn_reg <= 3'b000;
      adjlat_reg <= 1'b0;
      end
    else begin
      if (~|c10240_reg) begin
        adj_dn_reg <=  updown_reg && (adjust_pls || adjlat_reg);
        adj_up_reg <= !updown_reg && (adjust_pls || adjlat_reg);
        end
      adjsyn_reg <= {acctmrtc_reg, adjsyn_reg[2:1]};
      adjlat_reg <= |c10240_reg && (adjust_pls || adjlat_reg);
      end
    end

  assign hz100_pls = !c100syn_reg[1] && c100syn_reg[0];

  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) begin
      c100syn_reg <= 3'b000;
      c100lat_reg <= 1'b0;
      hz100_reg   <= 1'b0;
      end
    else begin
      c100syn_reg <= {c100_pls, c100syn_reg[2:1]};
      c100lat_reg <= !ld_inst && (hz100_pls || c100lat_reg);
      if (ld_inst) hz100_reg <= hz100_pls || c100lat_reg;
      end
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* service request                                                                       */
  /*                                                                                       */
  /*****************************************************************************************/
  assign srvc_drv = state_d && enabl_reg;
  assign srvc_req = alma_reg || almb_reg || dtza_reg || dtzb_reg || dtzit_reg;

  assign isa_drv  = !pwo && !pus_reg && srvc_req;
  assign fi_drv   =  pwo && !pus_reg && awake_reg &&
                     ((state_b && (srvc_req || cycle_reg[3])) || srvc_drv);
  assign fi_out   =  pwo && !pus_reg && awake_reg && ((state_b && cycle_reg[3]) ||
                     (srvc_drv && (!srvc_req || cycle_reg[3])));

  /*****************************************************************************************/
  /*                                                                                       */
  /* data bus                                                                              */
  /*                                                                                       */
  /*****************************************************************************************/
  assign data_drv = cregn_reg && !ph2m;
  assign data_out = data_cha || data_chb;

  /*****************************************************************************************/
  /*                                                                                       */
  /* instruction register and decodes                                                      */
  /*                                                                                       */
  /*****************************************************************************************/
  assign rdclk_inst  = cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0000);
  assign rdhclk_inst = cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0001);
  assign rdalrm_inst = cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0010);
  assign rdacc_inst  = cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0011) && !ptr_reg;
  assign rdscr_inst  = cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0100);
  assign rdtmr_inst  = cregn_reg && ~|(lftch_reg[9:6] ^ 4'b0101);

  assign wrclk_inst  = regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0000);
  assign wrcclk_inst = regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0001);
  assign wralrm_inst = regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0010);
  assign wracc_inst  = regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0011) && !ptr_reg;
  assign wrscr_inst  = regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0100);
  assign wrtmr_inst  = regnc_reg && ~|(lftch_reg[9:6] ^ 4'b0101);
  assign writim_inst = regnc_reg && ~|(lftch_reg[9:8] ^ 2'b01) && lftch_reg[6];
  assign wralm_inst  = regnc_reg && ~|(lftch_reg[9:7] ^ 3'b101);
  assign wrstst_inst = regnc_reg && ~|(lftch_reg[9:7] ^ 3'b110);

  /*****************************************************************************************/
  /*                                                                                       */
  /* read data a                                                                           */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (lftch_reg or ptr_reg or aalrm_reg or aclk_reg or ascr_reg or state_ec801 or
            astatus_out or tmr_reg) begin
    casex ({lftch_reg[8:6], ptr_reg})
      4'b00x1: data_cha = aclk_reg[0];                     /* clock register a             */
      4'b0101: data_cha = aalrm_reg[0];                    /* alarm register a             */
      4'b0111: data_cha = astatus_out;                     /* status                       */
      4'b1001: data_cha = ascr_reg[0];                     /* scratch register a           */
      4'b101x: data_cha = tmr_reg[0] && state_ec801;       /* interval timer               */
      default: data_cha = 1'b0;
      endcase
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* status register a                                                                     */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) begin
      alaen_reg <= 1'b0;
      ckaen_reg <= 1'b0;
      iten_reg  <= 1'b0;
      end
    else begin
      if (ld_inst && ptr_reg &&  wralm_inst) alaen_reg <=  lftch_reg[6];
      if (ld_inst && ptr_reg && wrstst_inst) ckaen_reg <=  lftch_reg[6];
      if (ld_inst &&            writim_inst) iten_reg  <= !lftch_reg[7];
      end
    end

  assign astatus_out = (state_e && cycle_reg[0] && alma_reg) ||
                       (state_e && cycle_reg[1] && dtza_reg) ||
                       (state_c && cycle_reg[0] && dtzit_reg) ||
                       (state_c && cycle_reg[1] && pus_reg) ||
                       (state_c && cycle_reg[2] && ckaen_reg) ||
                       (state_8 && cycle_reg[0] && alaen_reg) ||
                       (state_8 && cycle_reg[2] && iten_reg);

  /*****************************************************************************************/
  /*                                                                                       */
  /* interval timer                                                                        */
  /*                                                                                       */
  /*****************************************************************************************/
  assign en_tmr    = (rdtmr_inst || wrtmr_inst || (iten_reg && hz100_reg)) && state_ec801;
  assign en_tmatch = ld_inst || (ld_cycle && iten_reg && state_ec801);
  assign tmr_nxt   = (wrtmr_inst) ? data_reg : tmr_reg[0];
  assign itmr_tc   = !writim_inst && !itmrtc_reg &&
                     ~|({itmr_nxt, itmr_reg[17]} ^ {tmr_reg[0], tmr_reg[19:17]});

  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) begin
      tmr_reg    <= 20'h00000;
      tmatch_reg <= 1'b0;
      itmrtc_reg <= 1'b0;
      dtzit_reg  <= 1'b0;
      end
    else begin
      if (en_tmr)    tmr_reg    <= {tmr_nxt, tmr_reg[19:1]};
      if (en_tmatch) tmatch_reg <= ld_inst || itmr_tc;
      if (ld_inst)   itmrtc_reg <=  iten_reg && hz100_reg && tmatch_reg;
      if (ld_inst)   dtzit_reg  <= (iten_reg && hz100_reg && tmatch_reg) ||
                                   (!dtzit_res && dtzit_reg);
      end
    end

  assign en_itmr  = (writim_inst || itmrtc_reg || (iten_reg && hz100_reg)) && state_ec801;
  assign itmr_inc = itmr_reg[0] ^  itmr_cry_reg;
  assign itmr_cry = itmr_reg[0] && itmr_cry_reg;
  assign itmr_nxt = (writim_inst) ? 3'b000 :
                    (itmrtc_reg)  ? {(hz100_reg && t0_reg), itmr_reg[19:18]} :
                    (ld_cycle)    ? itmrdadj_out : {itmr_inc, itmr_reg[19:18]};

  always @ (itmr_cry or itmr_inc or itmr_reg) begin
    casex ({itmr_cry, itmr_inc, itmr_reg[19:18]})
      4'bx101: itmrdadj_out = 3'b000;                      /* for inc operation            */
      4'bx110: itmrdadj_out = 3'b001;                      /* if result >= 1010            */ 
      4'bx111: itmrdadj_out = 3'b010;                      /* then sub 1010 from result    */
      4'b1000: itmrdadj_out = 3'b011;
      default: itmrdadj_out = {itmr_inc, itmr_reg[19:18]}; /* else no change               */
      endcase
    end

  assign itmrdadj_cry = itmr_cry || (itmr_inc && |itmr_reg[19:18]);

  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) begin
      itmr_cry_reg <= 1'b0;
      itmr_reg     <= 20'h00000;
      end
    else begin
      itmr_cry_reg <= (ld_cycle) ? ((iten_reg && ld_inst) || itmrdadj_cry) : itmr_cry;
      if (en_itmr) itmr_reg <= {itmr_nxt, itmr_reg[17:1]};
      end
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* scratch a register                                                                    */
  /*                                                                                       */
  /*****************************************************************************************/
  assign en_ascr  = ptr_reg && (rdscr_inst || wrscr_inst);
  assign ascr_nxt = (wrscr_inst) ? data_reg : ascr_reg[0];

  always @ (posedge ph2m or negedge pwr_up) begin
    if      (!pwr_up) ascr_reg <= 56'h00000000000000;
    else if (en_ascr) ascr_reg <= {ascr_nxt, ascr_reg[55:1]};
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* alarm/clock a                                                                         */
  /*                                                                                       */
  /*****************************************************************************************/
  assign en_aalrm  = (ptr_reg && (rdalrm_inst || wralrm_inst)) || (a_hz100 && !aclkhld_reg);
  assign en_amatch = ld_inst || (ld_cycle && alaen_reg && ckaen_reg);
  assign aalrm_nxt = (wralrm_inst) ? data_reg : aalrm_reg[0];
  assign aclk_mat  = !wr_aclk && !rd_aonly &&
                     ~|({aclk_nxt, aclk_reg[53]} ^ {aalrm_reg[0], aalrm_reg[55:53]});

  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) begin
      aalrm_reg  <= 56'h00000000000000;
      amatch_reg <= 1'b0;
      alma_reg   <= 1'b0;
      end
    else begin
      if (en_aalrm)  aalrm_reg  <= {aalrm_nxt, aalrm_reg[55:1]};
      if (en_amatch) amatch_reg <= alaen_reg && ckaen_reg && (ld_inst || aclk_mat);
      if (ld_inst)   alma_reg   <= (aclk_mat && amatch_reg) || (!alma_res && alma_reg);
      end
    end

  assign a_hz100  = ckaen_reg && hz100_reg;
  assign rd_aclk  = ptr_reg && (rdhclk_inst || rdclk_inst);
  assign rd_ahclk = ptr_reg &&  rdhclk_inst;
  assign rd_aonly = rd_ahclk || aclkhld_reg || (rd_aclk && !a_hz100);
  assign wr_aclk  = ptr_reg && (wrcclk_inst || wrclk_inst);
  assign wr_acclk = ptr_reg &&  wrcclk_inst;

  assign aclk_en  = (a_hz100 && !aclkhld_reg) || rd_aclk || wr_aclk;
  assign aclk_inc = aclk_reg[0] ^  aclk_cry_in;
  assign aclk_cry = aclk_reg[0] && aclk_cry_in;

  assign aclk_nxt = (wr_aclk)  ? {data_reg,    aclk_reg[55:54]} :
                    (rd_aonly) ? {aclk_reg[0], aclk_reg[55:54]} :
                    (ld_cycle) ?  aclkdadj_out :
                                 {aclk_inc,    aclk_reg[55:54]};

  assign aclk_0cry   = wr_aclk || rd_aonly || aclkhld_reg || !a_hz100 ||
                       (aclkinc2_reg && t0_reg); 
  assign aclk_1cry   = (!aclkinc2_reg && !aclkhld_reg && a_hz100 && t0_reg) ||
                       ( aclkinc2_reg && !aclkhld_reg && a_hz100 && t1_reg);
  assign aclk_cry_in = !aclk_0cry && (aclk_1cry || aclk_cry_reg);

  always @ (aclk_cry or aclk_inc or aclk_reg) begin
    casex ({aclk_cry, aclk_inc, aclk_reg[55:54]})
      4'bx101: aclkdadj_out = 3'b000;                      /* for inc operation            */
      4'bx110: aclkdadj_out = 3'b001;                      /* if result >= 1010            */ 
      4'bx111: aclkdadj_out = 3'b010;                      /* then sub 1010 from result    */
      4'b1000: aclkdadj_out = 3'b011;
      default: aclkdadj_out = {aclk_inc, aclk_reg[55:54]}; /* else no change               */
      endcase
    end

  assign aclkdadj_cry = aclk_cry || (aclk_inc && |aclk_reg[55:54]);

  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) begin
      aclk_cry_reg <= 1'b0;
      aclk_reg     <= 56'h00000000000000;
      aclkhld_reg  <= 1'b0;
      aclkinc2_reg <= 1'b0;
      dtza_reg     <= 1'b0;
      end
    else begin
      aclk_cry_reg          <= (ld_cycle) ? aclkdadj_cry : aclk_cry;
      if (aclk_en) aclk_reg <= {aclk_nxt, aclk_reg[53:1]};
      if (ld_inst) begin
        aclkhld_reg  <= !wr_acclk && (rd_ahclk || aclkhld_reg);
        aclkinc2_reg <= (a_hz100  && (rd_ahclk || aclkhld_reg)) ||
                        (!a_hz100 && aclkinc2_reg);
        dtza_reg     <= (ckaen_reg && aclkdadj_cry) || (!dtza_res && dtza_reg);
        end
      end
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* read data b                                                                           */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (lftch_reg or ptr_reg or balrm_reg or bclk_reg or bscr_reg or correct_out or
            bstatus_out) begin
    casex ({lftch_reg[8:6], ptr_reg})
      4'b00x0: data_chb = bclk_reg[0];                     /* clock register b             */
      4'b0100: data_chb = balrm_reg[0];                    /* alarm register b             */
      4'b0110: data_chb = correct_out;                     /* accuracy factor              */
      4'b0111: data_chb = bstatus_out;                     /* status                       */
      4'b1000: data_chb = bscr_reg[0];                     /* scratch register b           */
      default: data_chb = 1'b0;
      endcase
    end

  assign bstatus_out = (state_e && cycle_reg[2] && almb_reg) ||
                       (state_e && cycle_reg[3] && dtzb_reg) ||
                       (state_c && cycle_reg[3] && ckben_reg) ||
                       (state_8 && cycle_reg[1] && alben_reg);

  /*****************************************************************************************/
  /*                                                                                       */
  /* status register b                                                                     */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) begin
      alben_reg <= 1'b0;
      ckben_reg <= 1'b0;
      end
    else begin
      if (ld_inst && !ptr_reg &&  wralm_inst) alben_reg <=  lftch_reg[6];
      if (ld_inst && !ptr_reg && wrstst_inst) ckben_reg <=  lftch_reg[6];
      end
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* accuracy factor register                                                              */
  /*                                                                                       */
  /*****************************************************************************************/
  assign en_acc      = (rdacc_inst || wracc_inst || (auto_adj && hz100_reg)) &&
                       (state_c80 || (state_1 && cycle_reg[0]));
  assign en_accmatch = ld_inst || (ld_cycle && auto_adj && state_c80);
  assign acc_nxt     = (wracc_inst) ? data_reg : acc_reg[0];
  assign acctmr_tc   = !wracc_inst && !acctmrtc_reg &&
                       ~|({acctmr_nxt, acctmr_reg[13]} ^ {acc_reg[0], acc_reg[12:10]});

  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) begin
      acc_reg      <= 13'h0000;
      accmatch_reg <= 1'b0;
      acctmrtc_reg <= 1'b0;
      updown_reg   <= 1'b0;
      end
    else begin
      if (en_acc)      acc_reg      <= {acc_nxt, acc_reg[12:1]};
      if (en_accmatch) accmatch_reg <= ld_inst || acctmr_tc;
      if (ld_inst)     acctmrtc_reg <= auto_adj && hz100_reg && accmatch_reg;
      if (ld_inst)     updown_reg   <= acc_reg[12];
      end
    end

  assign correct_out = en_acc && acc_reg[0];
  assign auto_adj    = |acc_reg;

  /*****************************************************************************************/
  /*                                                                                       */
  /* accuracy timer                                                                        */
  /*                                                                                       */
  /*****************************************************************************************/
  assign en_acctmr  = ((auto_adj && hz100_reg) || acctmrtc_reg) && state_ec80;
  assign acctmr_inc = acctmr_reg[0] ^  acctmr_cry_reg;
  assign acctmr_cry = acctmr_reg[0] && acctmr_cry_reg;
  assign acctmr_nxt = (acctmrtc_reg) ? {(hz100_reg && t0_reg), acctmr_reg[15:14]} :
                      (ld_cycle)     ? acctmrdadj_out : {acctmr_inc, acctmr_reg[15:14]};

  always @ (acctmr_cry or acctmr_inc or acctmr_reg) begin
    casex ({acctmr_cry, acctmr_inc, acctmr_reg[15:14]})
      4'bx101: acctmrdadj_out = 3'b000;                    /* for inc operation            */
      4'bx110: acctmrdadj_out = 3'b001;                    /* if result >= 1010            */ 
      4'bx111: acctmrdadj_out = 3'b010;                    /* then sub 1010 from result    */
      4'b1000: acctmrdadj_out = 3'b011;
      default: acctmrdadj_out = {acctmr_inc, acctmr_reg[15:14]};
      endcase
    end

  assign acctmrdadj_cry = acctmr_cry || (acctmr_inc && |acctmr_reg[15:14]);

  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) begin
      acctmr_cry_reg <= 1'b0;
      acctmr_reg     <= 16'h0000;
      end
    else begin
      acctmr_cry_reg <= (ld_cycle) ? ((auto_adj && ld_inst) || acctmrdadj_cry) : acctmr_cry;
      if (en_acctmr) acctmr_reg <= {acctmr_nxt, acctmr_reg[13:1]};
      end
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* scratch b register                                                                    */
  /*                                                                                       */
  /*****************************************************************************************/
  assign en_bscr  = !ptr_reg && (rdscr_inst || wrscr_inst);
  assign bscr_nxt = (wrscr_inst) ? data_reg : bscr_reg[0];

  always @ (posedge ph2m or negedge pwr_up) begin
    if      (!pwr_up) bscr_reg <= 56'h00000000000000;
    else if (en_bscr) bscr_reg <= {bscr_nxt, bscr_reg[55:1]};
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* alarm/clock b                                                                         */
  /*                                                                                       */
  /*****************************************************************************************/
  assign en_balrm  = (!ptr_reg && (rdalrm_inst || wralrm_inst)) || (b_hz100 && !bclkhld_reg);
  assign en_bmatch = ld_inst || (ld_cycle && alben_reg && ckben_reg);
  assign balrm_nxt = (wralrm_inst) ? data_reg : balrm_reg[0];
  assign bclk_mat  = !wr_bclk && !rd_bonly &&
                     ~|({bclk_nxt, bclk_reg[53]} ^ {balrm_reg[0], balrm_reg[55:53]});

  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) begin
      balrm_reg  <= 56'h00000000000000;
      bmatch_reg <= 1'b0;
      almb_reg   <= 1'b0;
      end
    else begin
      if (en_balrm)  balrm_reg  <= {balrm_nxt, balrm_reg[55:1]};
      if (en_bmatch) bmatch_reg <= alben_reg && ckben_reg && (ld_inst || bclk_mat);
      if (ld_inst)   almb_reg   <= (bclk_mat && bmatch_reg) || (!almb_res && almb_reg);
      end
    end

  assign b_hz100  = ckben_reg && hz100_reg;
  assign rd_bclk  = !ptr_reg && (rdhclk_inst || rdclk_inst);
  assign rd_bhclk = !ptr_reg &&  rdhclk_inst;
  assign rd_bonly = rd_bhclk || bclkhld_reg || (rd_bclk && !b_hz100);
  assign wr_bclk  = !ptr_reg && (wrcclk_inst || wrclk_inst);
  assign wr_bcclk = !ptr_reg &&  wrcclk_inst;

  assign bclk_en  = (b_hz100 && !bclkhld_reg) || rd_bclk || wr_bclk;
  assign bclk_inc = bclk_reg[0] ^  bclk_cry_in;
  assign bclk_cry = bclk_reg[0] && bclk_cry_in;

  assign bclk_nxt = (wr_bclk)  ? {data_reg,    bclk_reg[55:54]} :
                    (rd_bonly) ? {bclk_reg[0], bclk_reg[55:54]} :
                    (ld_cycle) ?  bclkdadj_out :
                                 {bclk_inc,    bclk_reg[55:54]};

  assign bclk_0cry   = wr_bclk || rd_bonly || bclkhld_reg || !b_hz100 ||
                       (bclkinc2_reg && t0_reg); 
  assign bclk_1cry   = (!bclkinc2_reg && !bclkhld_reg && b_hz100 && t0_reg) ||
                       ( bclkinc2_reg && !bclkhld_reg && b_hz100 && t1_reg);
  assign bclk_cry_in = !bclk_0cry && (bclk_1cry || bclk_cry_reg);

  always @ (bclk_cry or bclk_inc or bclk_reg) begin
    casex ({bclk_cry, bclk_inc, bclk_reg[55:54]})
      4'bx101: bclkdadj_out = 3'b000;                      /* for inc operation            */
      4'bx110: bclkdadj_out = 3'b001;                      /* if result >= 1010            */ 
      4'bx111: bclkdadj_out = 3'b010;                      /* then sub 1010 from result    */
      4'b1000: bclkdadj_out = 3'b011;
      default: bclkdadj_out = {bclk_inc, bclk_reg[55:54]}; /* else no change               */
      endcase
    end

  assign bclkdadj_cry = bclk_cry || (bclk_inc && |bclk_reg[55:54]);

  always @ (posedge ph2m or negedge pwr_up) begin
    if (!pwr_up) begin
      bclk_cry_reg <= 1'b0;
      bclk_reg     <= 56'h00000000000000;
      bclkhld_reg  <= 1'b0;
      bclkinc2_reg <= 1'b0;
      dtzb_reg     <= 1'b0;
      end
    else begin
      bclk_cry_reg          <= (ld_cycle) ? bclkdadj_cry : bclk_cry;
      if (bclk_en) bclk_reg <= {bclk_nxt, bclk_reg[53:1]};
      if (ld_inst) begin
        bclkhld_reg  <= !wr_bcclk && (rd_bhclk || bclkhld_reg);
        bclkinc2_reg <= (b_hz100  && (rd_bhclk || bclkhld_reg)) ||
                        (!b_hz100 && bclkinc2_reg);
        dtzb_reg     <= (ckben_reg && bclkdadj_cry) || (!dtzb_res && dtzb_reg);
        end
      end
    end

  endmodule








