/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2013, Systemyde International Corporation, ALL RIGHTS RESERVED          **/
/**                                                                                       **/
/** timer chip                                                        Rev 0.0  02/12/2013 **/
/**                                                                                       **/
/*******************************************************************************************/
module time_top (data_drv, data_out, fi_drv, fi_out, isa_drv, isa_out, data_bus, isa_bus,
                 osc_in, ph1, ph2, pwr_up, pwo, sync);

  input         data_bus;      /* memory/peripheral data bus                          bus  */
  input         isa_bus;       /* instruction address/data bus                        bus  */
  input         osc_in;        /* 32768Hz clock input                               32khz  */
  input         ph1;           /* clock phase 1                                       bus  */
  input         ph2;           /* clock phase 2                                       bus  */
  input         pwr_up;        /* power on reset                                      por  */
  input         pwo;           /* power off                                           bus  */
  input         sync;          /* synchronization                                     bus  */

  output        data_drv;      /* memory/peripheral data bus                          bus  */
  output        data_out;      /* memory/peripheral data bus                          bus  */
  output        fi_drv;        /* flag bus                                            bus  */
  output        fi_out;        /* flag bus                                            bus  */
  output        isa_drv;       /* instruction address/data bus                        bus  */
  output        isa_out;       /* instruction address/data bus                        bus  */

  /*****************************************************************************************/
  /*                                                                                       */
  /* signal declarations                                                                   */
  /*                                                                                       */
  /*****************************************************************************************/
  wire        data_drv;                                    /* memory/peripheral data bus   */
  wire        data_out;                                    /* memory/peripheral data bus   */
  wire        fi_drv;                                      /* flag bus                     */
  wire        fi_out;                                      /* flag bus                     */
  wire        isa_drv_c;                                   /* drive isa_bus (core)         */
  wire        isa_drv_r;                                   /* drive isa_bus (rom)          */
  wire        isa_out_r;                                   /* data: isa_bus (rom)          */

  /*****************************************************************************************/
  /*                                                                                       */
  /* bus drivers                                                                           */
  /*                                                                                       */
  /*****************************************************************************************/
  assign isa_drv  = isa_drv_c || isa_drv_r;
  assign isa_out  = isa_drv_c || isa_out_r;

  /*****************************************************************************************/
  /*                                                                                       */
  /* fpga                                                                                  */
  /*                                                                                       */
  /*****************************************************************************************/
  tim_core CORE ( .data_drv(data_drv), .data_out(data_out), .isa_drv(isa_drv_c),
                  .fi_drv(fi_drv), .fi_out(fi_out), .data_bus(data_bus), .isa_bus(isa_bus),
                  .osc_in(osc_in), .ph1(ph1), .ph2(ph2), .pwr_up(pwr_up), .pwo(pwo),
                  .sync(sync) );

  rom_top  ROM  ( .isa_drv(isa_drv_r), .isa_out(isa_out_r), .isa_bus(isa_bus), .ph1(ph1),
                  .ph2(ph2), .pwo(pwo), .sync(sync) );

  endmodule








